' ILI9325 Driver using Dracblade P0 - P11 pins and three latches
' and also contains the dracblade ram driver for fast block moves from ram to the display
' Methods:
' .Start = start the driver
' .Draw  = start drawing at x1,y1 and continue until x2,y2
' .Pixel = a pixel in 2 bytes RRRRRGGG GGGBBBBB
con
  RS                = 16            ''PINS FOR CS - TO KEEP FROM FLOATING 
  LCD_WR            = 17                         ''and sd card defs
  DEV_SD_CLK        = 18
  DEV_SD_DI         = 19
  DEV_SD_DO         = 20    
  DEV_SPI_EN        = 21
  DEV_SPI_ADD0      = 22
  DEV_SPI_ADD1      = 23                 
  DEV_SPI_ADD_LE    = 24
  
DEV_SPI_LCD_RES      = 0
DEV_SPI_SDC_CS       = 1
DEV_SPI_TSC_CS       = 2
DEV_SPI_EXT_CS       = 3
 
 DEV_SPI_EXT_POT_CS   = 0
 DEV_SPI_EXT_SW_CS    = 1                                                   
''LCD REGISTERS   
  REG_OSCILLATOR                = $0000                               ''Oscillator                                (R00h) (POR = 0000h)
  REG_DRIVEROUTPUTCONTROL       = $0001                               ''Driver Output Control                     (R01h) (POR = 2B3Fh)
  REG_LCDDRIVINGWAVFORM         = $0002                               ''LCD-Driving-Waveform Control              (R02h) (POR = 0000h)
  REG_POWERCONTROL1             = $0003                               ''                                          (R03H)
  REG_DISPLAYCONTROL            = $0007                               ''Display Control                           (R07h) (POR = 0000h)
  REG_FRAMECYCLECONTROL         = $000B                               ''Frame Cycle Control                       (R0Bh) (POR = 5308h)D308 BY DATASHEET
  REG_POWERCONTROL2             = $000C                               ''                                          (R0Ch) (POR = 0004)
  REG_POWERCONTROL3             = $000D                               ''                                          
  REG_POWERCONTROL4             = $000E                               ''
  REG_GATESCANPOSITION          = $000F                               ''Gate Scan Position                        (R0Fh) (POR = 0000h)
  REG_SLEEPMODE                 = $0010                               ''Sleep mode                                (R10h) (POR = 0001h)
  REG_ENTRYMODE                 = $0011                               ''Entry Mode                                (R11h) (POR = 6830h)             
  REG_HPORCH                    = $0016                               ''                                          (R16h) (POR = EF1Ch)
  REG_VPORCH                    = $0017                               ''                                          (R17h) (POR = 0003h)
  REG_POWERCONTROL5             = $001E                               ''                                          
  REG_RAMDATAWRITE              = $0022                               ''
  REG_RAMWRITEDATAMASK1         = $0023                               ''                                          (R23h) (POR = 0000h)
  REG_RAMWRITEDATAMASK2         = $0024                               ''                                          (R24h) (POR = 0000h)
  REG_VERTICALSCROLCONTROL1     = $0041                               ''                                          (R41h) (POR = 0000h)
  REG_VERTICALSCROLCONTROL2     = $0042                               ''                                          (R42h) (POR = 0000h)
  REG_HORIZONTALRAMADDRESSPOS   = $0044                               ''                                          (R44h) (POR = EF00h)
  REG_VERTICALRAMADDRESSSTART   = $0045                               ''                                          (R45h) (POR = 0000h)
  REG_VERTICALRAMADDRESSEND     = $0046                               ''                                          (R46h) (POR = 013Fh)
  REG_FIRSTWINDOWSTART          = $0048                               ''                                          (R48h) (POR = 0000h)
  REG_FIRSTWINDOWEND            = $0049                               ''                                          (R49h) (POR = 013Fh)
  REG_SECONDWINDOWSTART         = $004A                               ''                                          (R4Ah) (POR = 0000h)
  REG_SECONDWINDOWEND           = $004B                               ''                                          (R4Bh) (POR = 013Fh)
  REG_SETGDDRXADDRESSCOUNTER    = $004E                               ''                                          (R4Eh) (POR = 0000h)
  REG_SETGDDRYADDRESSCOUNTER    = $004F 
OBJ  
    wait    : "timing"                                    ' thanks to parallax   

VAR
     long DisplayPins ' status of all the display pins
     long orientation

 ' pass orientation true = portrait, false = landscape
PUB Start_SSD1289                                        ''Init Display

    EnableDisplayPins                                   ''enable pins 0 to 15
    WriteHigh
    LCD_Reset                                           ''reset screen
    wait.pause1ms(5)                                       
    LCD_Reset           ''then init screen
    wait.pause1ms(5)
    Lcd_Write_Com (REG_DISPLAYCONTROL)                  ''Display Control (R07h) (POR = 0000h) 
    Lcd_Write_Data($0021)                               ''GON = 1 DTE = 0 D[1:0] = 01 
    Lcd_Write_Com (REG_OSCILLATOR)                      ''Oscillator (R00h) (POR = 0000h)
    Lcd_Write_Data($0001)                               ''Turn on oscillator
    Lcd_Write_Com (REG_DISPLAYCONTROL)                  ''Display Control (R07h) (POR = 0000h)
    Lcd_Write_Data($0023)                               ''GON = 1 DTE = 0 D[1:0] = 11
    Lcd_Write_Com (REG_SLEEPMODE)                       ''Sleep mode (R10h) (POR = 0001h)
    Lcd_Write_Data($0000)     ''                        ''exit sleep mode         
    Lcd_Write_Com (REG_DISPLAYCONTROL)                  ''Display Control (R07h) (POR = 0000h)
    Lcd_Write_Data($0033)                               ''GON = 1 DTE = 1 D[1:0] = 11
    Lcd_Write_Com (REG_ENTRYMODE)                       ''Entry Mode (R11h) (POR = 6830h)
    Lcd_Write_Data($6838)                               ''
    Lcd_Write_Com (REG_LCDDRIVINGWAVFORM)               ''LCD-Driving-Waveform Control (R02h) (POR = 0000h)($1000) 
    Lcd_Write_Data($1000)     ''                        ''
    Lcd_Write_Com (REG_GATESCANPOSITION)                ''Gate Scan Position (R0Fh) (POR = 0000h)  ($0000)
    Lcd_Write_Data($0000)     ''                        ''                                                        ''
    Lcd_Write_Com (REG_DRIVEROUTPUTCONTROL)             ''Driver Output Control (R01h) (POR = [0XXXX0X1]3Fh) 433f    ($633F) 
    Lcd_Write_Data($6B3F) ''                            ''                                                      
    Lcd_Write_Com (REG_FRAMECYCLECONTROL)               ''Frame Cycle Control (R0Bh) (POR = 5308h) ($5308)
    Lcd_Write_Data($5308)                               ''                           '
    Lcd_Write_Com (REG_DISPLAYCONTROL)                  ''Display Control (R07h) (POR = 0000h) ($0033)
    Lcd_Write_Data($0033)                               ''GON = 1 DTE = 1 D[1:0] = 11
    TristateDisplayPins
    
PRI EnableDisplayPins
    DIRA:=%00000000_01100000_11111111_11111111 ' , Reset, WR, RS and 16 data lines active    
PRI TristateDisplayPins ' tristate all pins -                                                 
    DIRA:=%00000000_01100000_00000000_00000000 ''
    
PRI LCD_Writ_Bus(V) 
    OUTA[15..0]  := V
    WriteLow                                            ' write pin low
    WriteHigh                                           ' toggle write pin
    
PRI Lcd_Write_Com(V)
    RSLow
    LCD_Writ_Bus(V)
    
PRI Lcd_Write_Data(V)
    RSHigh
    LCD_Writ_Bus(V)
    
PRI RSLow                                    
    outa[RS] := 0
    dira[RS] := 1
    
PRI RSHigh
    dira[RS] := 0
                   '
PRI WriteLow                                    
    outa[LCD_WR] := 0
    dira[LCD_WR] := 1
                             ' 
PRI WriteHigh
    dira[LCD_WR] := 0

PRI LCD_Reset 
    OUTA[DEV_SPI_ADD1..DEV_SPI_ADD0] := DEV_SPI_LCD_RES                              
    outa[DEV_SPI_EN] := 0         
    wait.pause1ms(5)                                  ''wait here if necessary                           ' 
    outa[DEV_SPI_EN] := 1
    
'PUB ChangeOrientation(n) ' pass true = portrait or false = landscape, changes global variable orientation in this object
'    EnableDisplayPins
'    orientation := n
'    if orientation
'      Lcd_Write_Com($0001,$0100)                        ' set SS and SM bit 0001 0100   portrait
'      Lcd_Write_Com$0003,$1030)                        ' set GRAM write direction and BGR=1. $0003 $1030  
'    else
'     Lcd_Write_Com($0001,$0000)                        ' set SS and SM bit 0001 0000   landscape   
'     Lcd_Write_Com($0003,$1038)                        ' landscape  $1028 = original but 1038 is correct - not mirror image'   TristateDisplayPins

PUB Draw(x1, y1, x2, y2) | HORIZONTALRAMADDRESSPOS ' sets the pixel to x1,y1 and then fills the next (x2-x1)*(y2-y1) pixels
    EnableDisplayPins                       ' enable pins 0-11
    'ifnot orientation                                  ' landscape mode so swap x and y
    ' result :=x1                        ' swap x1 and y1
     ' x1 := y1
     ' y1 := result
    ' result := x2                       ' swap x2 and y2
     ' x2 :=y2
     ' y2 := result
      
  HORIZONTALRAMADDRESSPOS := x1 + (x2 << 8)     
    Lcd_Write_Com(REG_HORIZONTALRAMADDRESSPOS)
    Lcd_Write_Data(HORIZONTALRAMADDRESSPOS)
    Lcd_Write_Com(REG_VERTICALRAMADDRESSSTART)
    Lcd_Write_Data(y1)

    Lcd_Write_Com(REG_VERTICALRAMADDRESSEND)
    Lcd_Write_Data(y2)
    Lcd_Write_Com(REG_SETGDDRxADDRESSCOUNTER)
    Lcd_Write_Data(x1 )
    Lcd_Write_Com(REG_SETGDDRyADDRESSCOUNTER)
    Lcd_Write_Data( y1 )
    
    TristateDisplayPins                                   ' tristate pins 0-11 so other drivers can use these pins

PUB Pixel(pixelcolor)               ' send out a pixel, high byte then low byte
    EnableDisplayPins
    Lcd_Write_Com($22)               ' enable pins 0-11    
    Lcd_Write_Data(pixelcolor)                        
    TristateDisplayPins                                   ' tristate pins 0-11 so other drivers can use these pins               
    
'********************** private methods *********************


PUB ReadRamByte(address)                                ' masks off all but a byte - used often
    docmd("R",@result,address,1)                        ' read just 1 byte
    result := result & %00000000_00000000_00000000_11111111 ' mask off byte so returns a guaranteed byte value
    return result

PUB WriteRamByte(value,address)                         ' write a byte to ram
    docmd("W",@value,address,1)

PUB ReadRamLong(address)                             ' read a long, stored LSB first
    docmd("R",@result,address,4)
    return result

PUB WriteRamLong(value,address)                         ' write a long to ram
    docmd("W",@value,address,4) 
      

CON
''Dracblade driver for talking to a ram chip via three latches
'' Modified code from Cluso's triblade
'' also includes commands to move blocks of data to the ILI9325 touchscreen display
' DoCmd(command_, hub_address, ram_address, block_length)
' I - initialise     
' W - Move block_length bytes from hub_address to ram_address
' R - Move block_length bytes from ram_address to hub_address
' A - Move block of 256 bytes from hub to external ram. Faster than W as middle latch not changing each byte. block_length ignored (always 256) 
' *** command A and B must start on 0,256,512,768 etc so only the lower 8 bytes change ***
' B - Move a block of 256 bytes from external ram to hub. Faster than R as middle latch not changing each byte
' C - move a block of data (block_length) from ram_address to the ILI9325 display (assumes Draw has been called in spin first)
' D - move a block of data (block_length) from hub_address to the ILI9325 display (assumes Draw has been called in spin first)



VAR

' communication params(5) between cog driver code - only "command" and "errx" are modified by the driver
   long  command, hubaddrs, ramaddrs, blocklen, errx, cog ' rendezvous between spin and assembly (can be used cog to cog)
'        command  = R, W, H I,D etc =0 when operation completed by cog
'        hubaddrs = hub address for data buffer
'        ramaddrs = ram address for data ($0000 to $FFFF)
'        blocklen = ram buffer length for data transfer
'        errx     = returns =0 (false=good), else <>0 (true & error code)
'        cog      = cog no of driver (set by spin start routine)
   
PUB start_ram : err_
' Initialise the Drac Ram driver. No actual changes to ram as the read/write routines handle this
  command := "I"
  cog := 1 + cognew(@tbp2_start, @command)
  if cog == 0
    err_ := $FF                 ' error = no cog
  else
    repeat while command        ' driver cog sets =0 when done
    err_ := errx                ' driver cog sets =0 if no error, else xx = error code

PUB stop_ram
   if cog
      cogstop(cog~ - 1)      

PUB DoCmd(command_, hub_address, ram_address, block_length) : err_
' Do the command: R, W, N, F, H,I,D
  hubaddrs := hub_address       ' hub address start
  ramaddrs := ram_address       ' ram address start
  blocklen := block_length      ' block length
  command  := command_          ' must be last !!
' Wait for command to complete and get status
  repeat while command          ' driver cog sets =0 when done
  err_ := errx                  ' driver cog sets =0 if no error, else xx = error code


DAT
'' +--------------------------------------------------------------------------+
'' | Dracblade Ram Driver (with grateful acknowlegements to Cluso)            |
'' +--------------------------------------------------------------------------+
                        org     0
tbp2_start    ' setup the pointers to the hub command interface (saves execution time later
                                      '  +-- These instructions are overwritten as variables after start
comptr                  mov     comptr, par     ' -|  hub pointer to command                
hubptr                  mov     hubptr, par     '  |  hub pointer to hub address            
ramptr                  add     hubptr, #4      '  |  hub pointer to ram address            
lenptr                  mov     ramptr, par     '  |  hub pointer to length                 
errptr                  add     ramptr, #8      '  |  hub pointer to error status           
cmd                     mov     lenptr, par     '  |  command  I/R/W/G/P/Q                  
hubaddr                 add     lenptr, #12     '  |  hub address                           
ramaddr                 mov     errptr, par     '  |  ram address                           
len                     add     errptr, #16     '  |  length                                
err                     nop                     ' -+  error status returned (=0=false=good) 


' Initialise hardware tristates everything and read/write set the pins
init                    mov     err, #0                  ' reset err=false=good
                        mov     dira,zero                ' tristate the pins

done                    wrlong  err, errptr             ' status  =0=false=good, else error x
                        wrlong  zero, comptr            ' command =0 (done)
' wait for a command (pause short time to reduce power)
pause
'                        mov     ctr, delay      wz      ' if =0 no pause
'              if_nz     add     ctr, cnt
'              if_nz     waitcnt ctr, #0                 ' wait for a short time (reduces power)
                        rdlong  cmd, comptr     wz      ' command ?
              if_z      jmp     #pause                  ' not yet
' decode command
                        cmp     cmd, #"R"       wz      ' R = read block from external ram to hub
              if_z      jmp     #rdblock
                        cmp     cmd, #"W"       wz      ' W = write block from hub to external ram
              if_z      jmp     #wrblock
                        cmp     cmd, #"C"       wz      ' C moves a block of data from ram to the display
              if_z      jmp     #extramtodisplay
                        cmp     cmd, #"D"       wz      ' D moves a block of data from hub to the display
              if_z      jmp     #hubtodisplay
                        cmp     cmd, #"A"       wz      ' A moves 256 bytes from hub to external ram
              if_z      jmp     #blockhubtoram
                        cmp     cmd, #"B"       wz      ' B moves 256 bytes from external ram to hub
              if_z      jmp     #blockramtohub
                        cmp     cmd, #"E"       wz      ' convert 3 byte .raw format to 2 byte .ili format - hub to hub
              if_z      jmp     #rawtoiliformat
                        cmp     cmd, #"F"       wz      ' draw a transparent icon, pass two locations in hub ram and len = number of pixels
              if_z      jmp     #drawicon
                        cmp     cmd, #"I"       wz      ' init
              if_z      jmp     #init     
                        mov     err, cmd                ' error = cmd (unknown command)
                        jmp     #done

' set all latches (replaces H command), pass value in third variable of the DoCmd

'setlatches              call #ram_open                  ' gets address value in 'address'
'                        call #lowmiddlehighlatch        ' set all the latches
'                        jmp  #init

'---------------------------------------------------------------------------------------------------------
'Memory Access Functions
                        ' read block from external ram to hub
rdblock                 call    #ram_open               ' get variables from hub variables
rdloop                  call    #read_byte              ' read byte from address into data_8
                        wrbyte  data_8,hubaddr          ' write data_8 to hubaddr ie copy byte to hub
                        add     hubaddr,#1              ' add 1 to hub address
                        add     ramaddr,#1              ' add 1 to ram address
                        djnz    len,#rdloop             ' loop until done
                        jmp     #init                   ' reinitialise


                        ' write block from hub to external ram
wrblock                 call    #ram_open
wrloop                  rdbyte  data_8, hubaddr         ' copy byte from hub
                        call    #write_byte             ' write byte from data_8 to address
                        add     hubaddr,#1              ' add 1 to hub address
                        add     ramaddr,#1              ' add 1 to ram address
                        djnz    len,#wrloop             ' loop until done
                        jmp     #init                   ' reinitialise

ram_open                rdlong  hubaddr, hubptr         ' get hub address
                        rdlong  ramaddr, ramptr         ' get ram address
                        rdlong  len, lenptr             ' get length
                        mov     err, #5                 ' err=5
ram_open_ret            ret



read_byte               call #lowmiddlehighlatch        ' set up latches 
                        call #read_memory_byte          ' read a byte into data_8
read_byte_ret           ret                        


  
read_memory_byte        mov dira,LatchDirection2        ' for reads so P0-P7 tristate till do read
                        mov outa,GateHigh               ' actually ReadEnable but they are the same
                        andn outa,GateHigh              ' set gate low
                        nop                             ' short delay to stabilise
                        nop
                        mov data_8, ina                 ' read SRAM
                        and data_8, #$FF                ' extract 8 bits
                        or  outa,GateHigh               ' set the gate high again
read_memory_byte_ret    ret



write_byte              call #lowmiddlehighlatch        ' set up latches
                        call #write_memory_byte
write_byte_ret          ret

write_memory_byte       mov outx,data_8                 ' get the byte to output
                        and outx, #$FF                  ' ensure upper bytes=0
                        or outx,WriteEnable             ' or with correct 138 address
                        mov outa,outx                   ' send it out
                        andn outa,GateHigh              ' set gate low
                        nop                             ' no nop doesn't work, one does, so put in two to be sure
                        nop                             ' another NOP
                        or outa,GateHigh                ' set it high again
write_memory_byte_ret   ret

LowMiddleHighLatch ' sets up the ram latches. 
                        call #lowlatchout
                        call #middlelatchout
                        call #highlatchout
LowMiddleHighLatch_ret  ret

' write to low, middle and high latches as subroutines. Pass 'ramaddr' which is a long. Extract xxxxxxxx_HHHHHHHH_MMMMMMMM_LLLLLLLL

LowLatchOut             mov outx,ramaddr                ' get the address into a temp variable
                        and outx,#$FF                   ' mask the low byte
                        or  outx,LowAddress             ' or with 138 low address
                        call #LatchOutput
LowLatchOut_ret         ret                        

MiddleLatchOut          mov outx,ramaddr                ' middle byte get the address into a temp variable
                        shr outx,#8                     ' shift right by 8 places
                        and outx,#$FF                   ' mask the low byte
                        or  outx,MiddleAddress          ' or with 138 middle address
                        call #LatchOutput
MiddleLatchOut_ret      ret                        

HighLatchOut            mov outx,ramaddr                ' middle byte get the address into a temp variable
                        shr outx,#16                    ' shift right by 16 places
                        and outx,#$FF                   ' mask the low byte
                        or  outx,HighAddress            ' or with 138 high address
                        call #LatchOutput                        
HighLatchOut_ret        ret

LatchOutput             mov dira,LatchDirection         ' set up the pins for programming latch chips
                        mov outa,outx                   ' send it out
                        andn outa,GateHigh              ' set gate low  nop not needed
                        or outa,GateHigh                ' set it high again 
LatchOutput_ret         ret

' ** move a block of 256 bytes from hub to external ram **

Blockhubtoram           call    #ram_open               ' get variables from hub variables
                        call    #lowmiddlehighlatch     ' set all three latches
                        mov     len,#256                ' do 256 times
Blockhubtoram_loop                           
                        rdbyte  data_8, hubaddr         ' copy byte from hub
                        call    #write_memory_byte      ' send out to the ram 
                        add     hubaddr,#1              ' add 1 to hub address
                        add     ramaddr,#1              ' add 1 to ram address
                        call    #lowlatchout            ' update the low latch
                        djnz    len,#blockhubtoram_loop ' loop until done
                        jmp     #init                    ' restart

' ** move a block of 256 bytes from external ram to hub **

Blockramtohub           call    #ram_open               ' get variables from hub variables
                        call    #lowmiddlehighlatch     ' set all three latches
                        mov     len,#256                ' do 256 times
Blockramtohub_loop                           
                        call    #read_memory_byte       ' read byte from address into data_8
                        wrbyte  data_8,hubaddr          ' write data_8 to hubaddr ie copy byte to hub
                        add     hubaddr,#1              ' add 1 to hub address
                        add     ramaddr,#1              ' add 1 to ram address
                        call    #lowlatchout            ' update the low latch
                        djnz    len,#blockramtohub_loop ' loop until done
                        jmp     #init                    ' restart





extramtodisplay ' send a block of data blocklen from external memory ramaddress to the ILI9325 display
                        call    #ram_open               ' gets ramaddr and len (ignore hubaddr)
                        shr     len,#1                  ' len /2 as 2 bytes per pixel   
                        call    #setCSlowRShigh         ' set pins for data output 
ililoop                 call    #readiliword            ' read ilihigh, ililow, increments ramaddr by 2
                        call    #ilidisplayword         ' sends ilihigh ililow to the display
                        djnz    len,#ililoop            ' loop until done
                        jmp     #init                   ' set pins to tristate

readiliword             ' read ilihigh and ililow from ramaddr, increments ramaddr by 2
                        call    #read_byte              ' read byte from ramaddr into data_8
                        mov     dira,latchdirection     ' enable for output (readmemorybyte leaves P0-P7 as inputs) 
                        mov     ilihigh,data_8
                        add     ramaddr,#1              ' add 1 to ram address 
                        call    #read_byte              ' read second byte from address into data_8
                        mov     dira,latchdirection     ' enable for output (readmemorybyte leaves P0-P7 as inputs) 
                        mov     ililow,data_8
                        add     ramaddr,#1              ' add 1 to ram address
readiliword_ret         ret                        

hubtodisplay            ' send a block of data blocklen from hubaddr to the display (bypasses external ram)
                        call    #ram_open               ' gets hubaddr and len (ignores ramaddress)
                        shr     len,#1                  ' len /2 as 2 bytes per pixel
                        call    #setCSlowRShigh         ' set pins for data output
hubdisploop             rdbyte  ilihigh, hubaddr        ' copy byte from hub
                        add     hubaddr,#1              ' add 1 to hub address
                        rdbyte  ililow, hubaddr         ' copy byte from hub  
                        add     hubaddr,#1              ' add 1 to hub address
                        call    #ilidisplayword         ' send these bytes to the display
                        djnz    len,#hubdisploop        ' loop until done
                        jmp     #init                   ' set pins to tristate
                        

setCSlowRShigh          mov     dira,latchdirection     ' enable for output
                        mov     outa,latch7data         ' sets CS low, RS high (RS high is data output
                        andn    outa,GateHigh           ' set gate low
                        or      outa,GateHigh           ' set it high again
setCSlowRShigh_ret      ret

ilidisplayword            ' pass ililow and ilihigh, sends out to the display
                        or      ililow,Latch5address    ' or with the 138
                        mov     outa,ililow             ' send out
                        andn    outa,GateHigh           ' set gate low
                        or      outa,GateHigh           ' set it high again
                        or      ilihigh,Latch6address   ' or with the 138
                        mov     outa,ilihigh            ' send out
                        andn    outa,GateHigh           ' set gate low
                        or      outa,GateHigh           ' set it high again
                        mov     outa,latch7writelow     ' set the write pin low
                        andn    outa,GateHigh           ' set gate low
                        or      outa,GateHigh           ' set it high again
                        mov     outa,latch7data         ' set the write pin high
                        andn    outa,GateHigh           ' set gate low
                        or      outa,GateHigh           ' set it high again
ilidisplayword_ret        ret
                        
                                                
RawtoILIformat          ' takes a .raw 3 byte RRRRRRRR GGGGGGGG BBBBBBBB and converts to 2 byte RRRRRGGG GGGBBBBB
                        ' pass hubaddr, ramaddr and len
                        ' hubaddr is source location, len is number of pixels
                        ' ramaddr is destination in hub (messy naming) and length is 2/3 of blocklength
                        call    #ram_open ' gets hubaddress, ramaddress and len (ignores ramaddress)
rawloop
                        rdbyte red,hubaddr
                        add hubaddr,#1
                        rdbyte green,hubaddr
                        add hubaddr,#1
                        rdbyte blue,hubaddr
                        add hubaddr,#1
                        call #rgbtoili
                        wrbyte ilihigh,ramaddr
                        add ramaddr,#1
                        wrbyte ililow,ramaddr
                        add ramaddr,#1
                        djnz    len,#rawloop            ' loop until done 
                        jmp     #init                   ' set pins to tristate

RGBtoILI                ' pass red,green, blue, returns ililow and ilihigh
                        shr     red,#3                  ' 000RRRRR 
                        shl     red,#3                  ' RRRRR000 
                        shr     green,#2                ' 00GGGGGG
                        mov     ilihigh,green           ' ilihigh = 00GGGGGG
                        shr     ilihigh,#3              ' ilihigh = 00000GGG
                        or      ilihigh,red             ' ilihigh = RRRRRGGG
                        and     green,#%00000111        ' 00000GGG
                        shl     green,#5                ' GGG00000
                        mov     ililow,green            ' ililow = GGG00000
                        shr     blue,#3                 ' blue = 000BBBBB
                        or      ililow,blue             ' ililow = GGGBBBBB
RGBtoILI_ret            ret

ILItoRGB                ' pass ilihigh RRRRRGGG and ililow GGGBBBBB returns RRRRR000 GGGGGG00 BBBBB000
                        mov     red,ilihigh             ' red = RRRRRGGG
                        and     red,#%11111000          ' red = RRRRR000
                        mov     green,ilihigh           ' green = RRRRRGGG
                        shl     green,#5                ' green = GGG00000
                        mov     i,ililow                ' i = GGGBBBBB
                        and     i,#%11100000            ' i = GGG00000
                        shr     i,#3                    ' i = 000GGG00
                        or      green,i                 ' green = GGGGGG00
                        mov     blue,ililow             ' blue = GGGBBBBB
                        shl     blue,#3                 ' blue = BBBBB000
ILItoRGB_ret            ret

                       

DrawIcon                ' pass hubaddr, ramaddr and len. 
                        ' hubaddr is location in ram of the picture (messy naming of the variable)
                        ' ramaddr is the location in ram of the mask (both sources are ram, not hub)
                        ' format of the picture is 2 bytes RRRRRGGG GGGBBBBB
                        ' format of the icon is R G B A and A = 0 for transparent and 255 for full opaque (transparent usually round the edges)
                        call    #ram_open ' gets hubaddr, ramaddr and len
                        mov     backgroundcount,hubaddr ' backgroundcount is the picture
                        mov     iconcount,ramaddr       ' iconcount is the icon
drawiconloop            mov     ramaddr,backgroundcount ' address for reading the picture
                        call    #readiliword            ' get ilihigh and ililow RRRRRGGG GGGBBBBB
                        mov     backgroundcount,ramaddr ' store new picture counter
                        call    #ilitoRGB               ' convert to red green blue
                        mov     ramaddr,iconcount       ' get icon location
                        call    #readiliword            ' returns ilihigh and ililow which are actually red and green
                        mov     r,ilihigh               ' get red
                        mov     g,ililow                ' get green
                        call    #readiliword            ' returns ilihigh and ililow which are actually blue and alpha             
                        mov     b,ilihigh               ' get blue
                        mov     a,ililow                ' get alpha
                        mov     iconcount,ramaddr       ' restore the icon pointer
                        cmp     a,#128          wc      ' test if background or foreground
              if_c      jmp     #drawiconfinish         ' if carry then this is background
                        mov     red,r                   ' move icon to foreground
                        mov     green,g
                        mov     blue,b
drawiconfinish          call    #rgbtoili               ' convert to ililow and ilihigh
                        call    #ilidisplayword         ' send out to the display
                        djnz    len,#drawiconloop       ' loop len times until finished
                        jmp     #init                   ' set pins to tristate

delay                   long    80                                    ' waitcnt delay to reduce power (#80 = 1uS approx)
ctr                     long    0                                     ' used to pause execution (lower power use) & byte counter
GateHigh                long    %00000000_00000000_00000001_00000000  ' HC138 gate high, all others must be low
Outx                    long    0                                     ' for temp use, same as n in the spin code
LatchDirection          long    %00000000_00000000_00001111_11111111 ' 138 active, gate active and 8 data lines active
LatchDirection2         long    %00000000_00000000_00001111_00000000 ' for reads so data lines are tristate till the read
LowAddress              long    %00000000_00000000_00000101_00000000 ' low address latch = xxxx010x and gate high xxxxxxx1
MiddleAddress           long    %00000000_00000000_00000111_00000000 ' middle address latch = xxxx011x and gate high xxxxxxx1
HighAddress             long    %00000000_00000000_00001001_00000000 ' high address latch = xxxx100x and gate high xxxxxxx1
'ReadEnable long    %00000000_00000000_00000001_00000000 ' /RD = xxxx000x and gate high xxxxxxx1
                                                        ' commented out as the same as GateHigh
WriteEnable             long    %00000000_00000000_00000011_00000000 ' /WE = xxxx001x and gate high xxxxxxx1
Zero                    long    %00000000_00000000_00000000_00000000 ' for tristating all pins
data_8                  long    0                                    ' so code compatability with zicog driver
i                       long    0                                    ' general purpose value
j                       long    0                                    ' general purpose value
a                       long    0                                    ' general purpose value
b                       long    0                                    ' general purpose value
r                       long    0                                    ' general purpose value
g                       long    0                                    ' general purpose value
                                 
' variables and constants for the ILI9325 display
Latch5address           long    %00000000_00000000_00001011_00000000 ' latch5 address
Latch6address           long    %00000000_00000000_00001101_00000000 ' latch6 address
Latch7data              long    %00000000_00000000_00001111_00010111 ' latch7 address 111x and gate high, reset high, cs low, read high, write high, rs high for data output, combines rshigh in spin into one line                   
Latch7WriteLow          long    %00000000_00000000_00001111_00010101 ' cs low, write low
ililow                  long    0                                    ' low data byte latch5
ilihigh                 long    0                                    ' high data byte latch6
red                     long    0                                    ' red, green blue variables
green                   long    0
blue                    long    0
backgroundcount         long    0
iconcount               long    0
                        fit     496      